@@ -64,6 +64,8 @@
 //! [`SpanData`]: trait.SpanData.html
 use std::fmt::Debug;
 
+#[cfg(feature = "registry")]
+use crate::filter::FilterId;
 use tracing_core::{field::FieldSet, span::Id, Metadata};
 
 /// A module containing a type map of span extensions.
@@ -128,8 +130,34 @@ pub trait LookupSpan<'a> {
         Some(SpanRef {
             registry: self,
             data,
+            #[cfg(feature = "registry")]
+            filter: FilterId::none(),
         })
     }
+
+    /// Registers a [`Filter`] for [per-layer filtering] with this
+    /// [`Subscriber`].
+    ///
+    /// The [`Filter`] can then use the returned [`FilterId`] to
+    /// [check if it previously enabled a span][check].
+    ///
+    /// # Panics
+    ///
+    /// If this `Subscriber` does not support [per-layer filtering].
+    ///
+    /// [`Filter`]: crate::layer::Filter
+    /// [per-layer filtering]: crate::layer::Layer#per-layer-filtering
+    /// [`Subscriber`]: tracing_core::Subscriber
+    /// [`FilterId`]: crate::filter::FilterId
+    /// [check]: SpanData::is_enabled_for
+    #[cfg(feature = "registry")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "registry")))]
+    fn register_filter(&mut self) -> FilterId {
+        panic!(
+            "{} does not currently support filters",
+            std::any::type_name::<Self>()
+        )
+    }
 }
 
 /// A stored representation of data associated with a span.
@@ -154,6 +182,23 @@ pub trait SpanData<'a> {
     /// The extensions may be used by `Layer`s to store additional data
     /// describing the span.
     fn extensions_mut(&self) -> ExtensionsMut<'_>;
+
+    /// Returns `true` if this span is enabled for the [per-layer filter][plf]
+    /// corresponding to the provided [`FilterId`].
+    ///
+    /// ## Default Implementation
+    ///
+    /// By default, this method assumes that the [`LookupSpan`] implementation
+    /// does not support [per-layer filtering][plf], and always returns `true`.
+    ///
+    /// [plf]: crate::layer::Layer#per-layer-filtering
+    /// [`FilterId`]: crate::filter::FilterId
+    #[cfg(feature = "registry")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "registry")))]
+    fn is_enabled_for(&self, filter: FilterId) -> bool {
+        let _ = filter;
+        true
+    }
 }
 
 /// A reference to [span data] and the associated [registry].
@@ -168,6 +213,9 @@ pub trait SpanData<'a> {
 pub struct SpanRef<'a, R: LookupSpan<'a>> {
     registry: &'a R,
     data: R::Data,
+
+    #[cfg(feature = "registry")]
+    filter: FilterId,
 }
 
 /// An iterator over the parents of a span, ordered from leaf to root.
@@ -177,6 +225,9 @@ pub struct SpanRef<'a, R: LookupSpan<'a>> {
 pub struct Scope<'a, R> {
     registry: &'a R,
     next: Option<Id>,
+
+    #[cfg(feature = "registry")]
+    filter: FilterId,
 }
 
 impl<'a, R> Scope<'a, R>
@@ -212,9 +263,25 @@ where
     type Item = SpanRef<'a, R>;
 
     fn next(&mut self) -> Option<Self::Item> {
-        let curr = self.registry.span(self.next.as_ref()?)?;
-        self.next = curr.parent_id().cloned();
-        Some(curr)
+        loop {
+            let curr = self.registry.span(self.next.as_ref()?)?;
+
+            #[cfg(feature = "registry")]
+            let curr = curr.with_filter(self.filter);
+            self.next = curr.data.parent().cloned();
+
+            // If the `Scope` is filtered, check if the current span is enabled
+            // by the selected filter ID.
+
+            #[cfg(feature = "registry")]
+            if !curr.is_enabled_for(self.filter) {
+                // The current span in the chain is disabled for this
+                // filter. Try its parent.
+                continue;
+            }
+
+            return Some(curr);
+        }
     }
 }
 
@@ -333,15 +400,69 @@ where
 
     /// Returns the ID of this span's parent, or `None` if this span is the root
     /// of its trace tree.
+    #[deprecated(
+        note = "this method cannot properly support per-layer filtering, and may \
+            return the `Id` of a disabled span if per-layer filtering is in \
+            use. use `.parent().map(SpanRef::id)` instead.",
+        since = "0.2.21"
+    )]
     pub fn parent_id(&self) -> Option<&Id> {
+        // XXX(eliza): this doesn't work with PLF because the ID is potentially
+        // borrowed from a parent we got from the registry, rather than from
+        // `self`, so we can't return a borrowed parent. so, right now, we just
+        // return the actual parent ID, and ignore PLF. which is not great.
+        //
+        // i think if we want this to play nice with PLF, we should just change
+        // it to return the `Id` by value instead of `&Id` (which we ought to do
+        // anyway since an `Id` is just a word) but that's a breaking change.
+        // alternatively, we could deprecate this method since it can't support
+        // PLF in its current form (which is what we would want to do if we want
+        // to release PLF in a minor version)...
+
+        // let mut id = self.data.parent()?;
+        // loop {
+        //     // Is this parent enabled by our filter?
+        //     if self
+        //         .filter
+        //         .map(|filter| self.registry.is_enabled_for(id, filter))
+        //         .unwrap_or(true)
+        //     {
+        //         return Some(id);
+        //     }
+        //     id = self.registry.span_data(id)?.parent()?;
+        // }
         self.data.parent()
     }
 
     /// Returns a `SpanRef` describing this span's parent, or `None` if this
     /// span is the root of its trace tree.
+
     pub fn parent(&self) -> Option<Self> {
         let id = self.data.parent()?;
         let data = self.registry.span_data(id)?;
+
+        #[cfg(feature = "registry")]
+        {
+            // move these into mut bindings if the registry feature is enabled,
+            // since they may be mutated in the loop.
+            let mut data = data;
+            loop {
+                // Is this parent enabled by our filter?
+                if data.is_enabled_for(self.filter) {
+                    return Some(Self {
+                        registry: self.registry,
+                        filter: self.filter,
+                        data,
+                    });
+                }
+
+                // It's not enabled. If the disabled span has a parent, try that!
+                let id = data.parent()?;
+                data = self.registry.span_data(id)?;
+            }
+        }
+
+        #[cfg(not(feature = "registry"))]
         Some(Self {
             registry: self.registry,
             data,
@@ -419,6 +540,9 @@ where
         Scope {
             registry: self.registry,
             next: Some(self.id()),
+
+            #[cfg(feature = "registry")]
+            filter: self.filter,
         }
     }
 
@@ -436,6 +560,9 @@ where
         Parents(Scope {
             registry: self.registry,
             next: self.parent_id().cloned(),
+
+            #[cfg(feature = "registry")]
+            filter: self.filter,
         })
     }
 
@@ -472,9 +599,30 @@ where
     pub fn extensions_mut(&self) -> ExtensionsMut<'_> {
         self.data.extensions_mut()
     }
+
+    #[cfg(feature = "registry")]
+    pub(crate) fn try_with_filter(self, filter: FilterId) -> Option<Self> {
+        if self.is_enabled_for(filter) {
+            return Some(self.with_filter(filter));
+        }
+
+        None
+    }
+
+    #[inline]
+    #[cfg(feature = "registry")]
+    pub(crate) fn is_enabled_for(&self, filter: FilterId) -> bool {
+        self.data.is_enabled_for(filter)
+    }
+
+    #[inline]
+    #[cfg(feature = "registry")]
+    fn with_filter(self, filter: FilterId) -> Self {
+        Self { filter, ..self }
+    }
 }
 
-#[cfg(test)]
+#[cfg(all(test, feature = "registry"))]
 mod tests {
     use crate::{
         layer::{Context, Layer},
